@@ -1,5 +1,5 @@
 package Gherkin::Token;
-$Gherkin::Token::VERSION = '23.0.1';
+$Gherkin::Token::VERSION = '24.0.0';
 use strict;
 use warnings;
 
@@ -7,7 +7,8 @@ use Class::XSAccessor
   constructor => 'new',
   accessors   => [
     qw/line location/,
-    map { "matched_$_" } qw/type keyword indent items text gherkin_dialect/
+    map { "matched_$_" } qw/type keyword keyword_type
+      indent items text gherkin_dialect/
   ],
   ;
 
